open Types ;;


type request =
  | Simple of string
  | And of request * request
  | Or of request * request
  | Followed_by of request * request
  | Contained_in of request * request   (* #1 contenu dans #2 *)
  | Sized of int
;;


let (+) x y =
  if y <> epsilon then assert false ;
  if x >= infty then infty
  else if x <= -infty then -infty
  else x + y
;;

let (-) x y =
  if y <> epsilon then assert false ;
  if x >= infty then infty
  else if x <= -infty then -infty
  else x - y
;;


(* ******************
Premier extent commenant  ou aprs k
*)
let rec (tau : index -> request -> position -> extent) =
  fun index request k ->
    if k >= infty then
      (Myformat.printf "tau@[ aborted@\nResult : (infty, infty)@]@\n" ;
       (infty, infty))
    else
    match request with
     | Simple str -> (* V *)
	 Myformat.printf "tau@[ Simple (%s), %d@\n" str k ;
	 let gc_list = Gclist.make_gc_list_for_string index str in
	 let res = Gclist.tau gc_list k in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Followed_by (a, b) -> (* V *)
	 Myformat.printf "tau@[ Followed_by, %d@\n" k ;
	 let (p, q) = tau index a k in
	 if p >= infty then
	   (Myformat.printf "aborted1@\nResult : (infty,infty)@]@\n" ;
	    (infty, infty))
	 else
	   let (p', q') = tau index b (q + epsilon) in
	   if p' >= infty then
	     (Myformat.printf "aborted2@\nResult : (infty,infty)@]@\n" ;
	      (infty, infty))
	   else
	     let (p'', q'') = tau' index a (p' - epsilon) in
	 Myformat.printf "Result : (%d, %d)@]@\n" p'' q' ;
	 (p'', q')
     | And (a, b) -> (* V *)
	 Myformat.printf "tau@[ And, %d@\n" k ;
	 let (p, q) = tau index a k in
	 if p >= infty then
	   (Myformat.printf "aborted1@\nResult : (infty,infty)@]@\n" ;
	    (infty, infty))
	 else
	   let (p', q') = tau index b k in
	   if p' >= infty then
	     (Myformat.printf "aborted2@\nResult : (infty,infty)@]@\n" ;
	      (infty, infty))
	   else
	     let (p'', q'') = tau' index a (max q q') in
	     let (p''', q''') = tau' index b (max q q') in
         (* Euh... *)
	 let res = ((min p'' p'''), (max q'' q''')) in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Or (a, b) -> (* V *)
	 Myformat.printf "tau@[ Or, %d@\n" k ;
	 let (p, q) = tau index a k in
	 let (p', q') = tau index b k in
	 let res = if q < q' then (p, q)
	 else if q > q' then (p', q')
	 else ((max p p'), q) in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Contained_in (a, b) -> (* V *)
	 Myformat.printf "tau@[ Contained_in, %d@\n" k ;
	 let (p, q) = tau index a k in
	 let (p', q') = rho index b q in
	 let res = if p' <= p then (p, q)
	           else tau index (Contained_in (a, b)) p' in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Sized n -> (* V *)
	 (k, k + n - epsilon)


(* ******************
Dernier extent terminant  ou avant k
*)
and (tau' : index -> request -> position -> extent) =
  fun index request k ->
    if k <= -infty then
      (Myformat.printf "tau'@[ aborted@\nResult : (-infty, -infty)@]@\n" ;
       (-infty, -infty))
    else
    match request with
     | Simple str -> (* V *)
	 Myformat.printf "tau'@[ Simple (%s), %d@\n" str k ;
	 let gc_list = Gclist.make_gc_list_for_string index str in
	 let res = Gclist.tau' gc_list k in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Followed_by (a, b) -> (* V *)
	 Myformat.printf "tau'@[ Followed_by, %d@\n" k ;
	 let (p, q) = tau' index b k in
	 if p <= -infty then
	   (Myformat.printf "aborted1@\nResult : (-infty,-infty)@]@\n" ;
	    (-infty, -infty))
	 else
           let (p', q') = tau' index a (p - epsilon) in
	   if p' <= -infty then
	     (Myformat.printf "aborted2@\nResult : (-infty,-infty)@]@\n" ;
	      (-infty, -infty))
	   else
	     let (p'', q'') = tau index b (q' + epsilon) in
	 Myformat.printf "Result : (%d, %d)@]@\n" p' q'' ;
	 (p', q'')
     | And (a, b) ->
	 Myformat.printf "tau'@[ And, %d@\n" k ;
	 let (p, q) = tau' index a k in
	 if p <= -infty then
	   (Myformat.printf "aborted1@\nResult : (-infty,-infty)@]@\n" ;
	    (-infty, -infty))
	 else
	   let (p', q') = tau' index b k in
	   if p' <= -infty then
	     (Myformat.printf "aborted2@\nResult : (-infty,-infty)@]@\n" ;
	      (-infty, -infty))
	   else
	     let (p'', q'') = tau index a (min p p') in
	     let (p''', q''') = tau index b (min p p') in
	     let res = ((min p'' p'''), (max q'' q''')) in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Or (a, b) ->
	 Myformat.printf "tau'@[ Or, %d@\n" k ;
	 let (p, q) = tau' index a k in
	 let (p', q') = tau' index b k in
	 let res = if p > p' then (p, q)
	 else if p < p' then (p', q')
	 else (p, min q q') in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Contained_in (a, b) ->
	 Myformat.printf "tau'@[ Contained_in, %d@\n" k ;
	 let (p, q) = tau' index a k in
	 let (p', q') = rho' index b p in
	 let res = if q' >= q then (p, q)
	 else tau' index (Contained_in (a, b)) q' in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Sized n -> (* V *)
	 rho index (Sized n) k



(* ******************
Premier extent terminant  ou aprs k
*)
and (rho : index -> request -> position -> extent) =
  fun index request k ->
    if k >= infty then
      (Myformat.printf "rho@[ aborted@\nResult : (infty, infty)@]@\n" ;
       (infty, infty))
    else
    match request with
     | Simple str -> (* V *)
	 Myformat.printf "rho@[ Simple (%s), %d@\n" str k ;
	 let gc_list = Gclist.make_gc_list_for_string index str in
	 let res = Gclist.rho gc_list k in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Followed_by (a, b) -> (* V *)
	 Myformat.printf "rho@[ Followed_by, %d@\n" k ;
	 let (p, q) = tau' index (Followed_by (a, b)) (k - epsilon) in
	 let res = tau index (Followed_by (a, b)) (p + epsilon) in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | And (a, b) ->
	 Myformat.printf "rho@[ And, %d@\n" k ;
	 let (p, q) = tau' index (And (a, b)) (k - epsilon) in
	 let res = tau index (And (a, b)) (p + epsilon) in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Or (a, b) ->
	 Myformat.printf "rho@[ Or, %d@\n" k ;
	 let (p, q) = rho index a k in
	 let (p', q') = rho index b k in
	 let res = if q < q' then (p, q)
	 else if q > q' then (p', q')
	 else ((max p p'), q) in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Contained_in (a, b) ->
	 Myformat.printf "rho@[ Contained_in, %d@\n" k ;
	 let (p, q) = rho index a k in
	 let res = tau index (Contained_in (a, b)) p in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Sized n -> (* V *)
	 (k - n + epsilon, k)



(* ******************
Dernier extent commenant  ou avant k
*)
and (rho' : index -> request -> position -> extent) =
  fun index request k ->
    if k <= -infty then
      (Myformat.printf "rho'@[ aborted@\nResult : (-infty, -infty)@]@\n" ;
       (-infty, -infty))
    else
    match request with
     | Simple str ->
	 Myformat.printf "rho'@[ Simple (%s), %d@\n" str k ;
	 let gc_list = Gclist.make_gc_list_for_string index str in
	 let res = Gclist.rho' gc_list k in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | And (a, b) -> assert false
     | Or (a, b) ->
	 Myformat.printf "rho'@[ Or, %d@\n" k ;
	 let (p, q) = tau index (Or (a, b)) (k + epsilon) in
	 let res = tau' index (Or (a, b)) (q - epsilon) in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Followed_by (a, b) ->
	 Myformat.printf "rho'@[ Followed_by, %d@\n" k ;
	 let (p, q) = tau index (Followed_by (a, b)) (k + epsilon) in
	 let res = tau' index (Followed_by (a, b)) (q - epsilon) in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Contained_in (a, b) ->
	 Myformat.printf "rho'@[ Contained_in, %d@\n" k ;
	 let (p, q) = rho' index a k in
	 let res = tau' index (Contained_in (a, b)) q in
	 Myformat.printf "Result : (%d, %d)@]@\n" (fst res) (snd res) ;
	 res
     | Sized n -> (* V *)
	 tau index (Sized n) k
;;



let driver req index =
 let res_gc_list = ref [] in
 let current_pos = ref (0) in
 let go_on = ref true in
 while !go_on do
   if !debug then
     Myformat.printf "@\nSearching from position %d@\n" !current_pos ;
   let (p, q) = tau index req !current_pos in
   if p >= infty then go_on := false
   else
     begin
     if !debug then Myformat.printf "Found at position %d - %d@\n" p q ;
     res_gc_list := !res_gc_list @ [(p,q)] ;
     current_pos := p + epsilon
     end
 done ;
 !res_gc_list
;;

