(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


exception Found of Index.position_mapping ;;


(* Mmorisation de la dernire position dans la map des fichiers o on  *)
(* s'est arrt lors du prcdent appel  map_extend_on_file. Comme les *)
(* rsultats d'une requte nous arrivent en ordre croissant, a permet  *)
(* d'acclrer la recherche du mapping entre position globale et nom de *)
(* fichier. Par contre, en contrepartie il faut penser  rinitialiser  *)
(* cette variable entre deux requtes ! On utilisera la fonction        *)
(* 'map_extend_init' dfinie ci-dessous.                                *)
let last_found = ref 0 ;;



(* **************************
val map_extend_init : unit -> unit

Rinitialise le marqueur mmorisant o on s'est arrt lors du dernier appel
 'map_extend_on_file'.
************************** *)
let map_extend_init () =
  last_found := 0 ;;



(* **************************
val map_extend_on_file :
  Index.t -> Gclist.extent ->
  (Index.file_name * Index.file_position * Index.file_position)

Permet de retrouver,  partir d'une position de dbut et de fin dans la base
de donne globale, le nom du fichier correspondant ainsi que les positions de
dbut et de fin dans ce fichier (positions commenant  0).
************************** *)
let map_extend_on_file index (start, stop) =
  try
    let map = index.Index.map in
    for i = !last_found to Array.length map - 1 do
      let ((fstart, fstop), fname) = map.(i) in
      if fstart <= start && fstop >= stop then
	begin
	last_found := i ;
	raise (Found ((start - fstart, stop - fstart), fname))
	end
    done ;
    (* Ceci ne doit pas se produire avec le mcanisme d'infinis locaux *)
    failwith ("Out of file : "^(string_of_int start)^" "^(string_of_int stop))
  with Found location -> location
;;
