(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


type file_name = string ;;



type file_position = int ;;



type file_extent = (file_position * file_position) ;;



type position_mapping = (file_extent * file_name) ;;



type index_data = (Token.t, file_position array) Hashtbl.t ;;



type t = {
    stoplist : Stoplist.t ;
    map : position_mapping array ;
    data : index_data
  } ;;


type preprocess =
  | No_pp
  | External_pp of string
;;

let version_release = "Version 1.0.1" ;;



(* ***********************************************************************
   [Fonc] build_command_call : string -> string -> int -> string
   [Description] : Construit l'invocation de la commande de viewer externe
       partir du format de commande trouv dans les prfrences. Dans ce
      format, le caractre '*' est remplac par le nom du fichier 
      afficher et le caractre '@' est remplac par la position de dpart
      (numro de caractre de 0  longueur - 1) o se positionner dans le
      fichier.
   [Args] :
     - string : Le format contenant le nom de la commande  lancer.
     - string : Le nom du fichier argument.
     - int : Position en caractre ( partir de 0) o se positionner.
   [Retour] :
     - string : La ligne de commande complte, prte  tre passe  la
         primitive systme de lancement de programme.
   [Rem] : Exporte hors de ce module.
           Peut lever l'exception 'Assert' dans le cas o le format pass
             en argument est incorrect car il ne contient pas de '*'. Il
             est donc  la charge de l'appelant de s'assurer de la
             correction de son format au pralable.
   *********************************************************************** *)
let build_command_call command filename position =
  let result= ref "" in
  for i = 0 to (String.length command) -1 do
    match command.[i] with
     | '*' -> result := !result ^ filename
     | '@' -> result := !result ^ (string_of_int position)
     | whatever -> result := !result ^ (String.make 1 whatever)
    done ;
  !result
;;
