{
(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)

(* Attention : pour ocamllex '\xxx' est en dcimal alors *)
(* que /usr/share/magic les donne en octal. *)

type format =
  | Postscript
  | Pdf
  | Lha
  | Dvi
  | Latex
  | MS_Office_doc
  | Rar
  | Zip
  | Gzip
  | Compress
  | Arj
  | Msdos_exe
  | Gif
  | Png
  | Jpeg
  | Bmp
  | Xpm
  | Riff_wav
  | Mod
  | Realaudio
  | Iff
  | Fig
  | Phigs
  | Cgm
  | Sit
  | Rtf
  | Html
  | Tar
  | Java_bytecode
  | C_H
  | PBM_image_text
  | PGM_image_text
  | PPM_image_text
  | PBM_image_rawbits
  | PGM_image_rawbits
  | PPM_image_rawbits
  | PAM_image
  | ELF_32_bits_exe
  | ELF_64_bits_exe
  | OCAML_Cmi
  | OCAML_Cmo
  | OCAML_Cmx
  | OCAML_Cma
  | OCAML_Cmxa
  | Unknown_format
;;


let to_string = function
  | Postscript -> "Postscript"
  | Pdf -> "Pdf"
  | Lha -> "Lha"
  | Dvi -> "Dvi"
  | Latex -> "Latex"
  | MS_Office_doc -> "Microsoft Office document"
  | Rar -> "Rar"
  | Zip -> "Zip"
  | Gzip -> "Gzip"
  | Compress -> "Compress"
  | Arj -> "Arj"
  | Msdos_exe -> "Msdos executable"
  | Gif -> "Gif"
  | Png -> "Png"
  | Jpeg -> "Jpeg"
  | Bmp -> "Bmp"
  | Xpm -> "Xpm"
  | Riff_wav -> "Riff wav"
  | Mod -> "Mod"
  | Realaudio -> "Realaudio"
  | Iff -> "Iff"
  | Fig -> "Fig"
  | Phigs -> "Phigs"
  | Cgm -> "Cgm"
  | Sit -> "Sit"
  | Rtf -> "Rtf"
  | Html -> "Html"
  | Tar -> "Tar"
  | Java_bytecode -> "Java bytecode"
  | C_H -> ".c/.h"
  | PBM_image_text -> "Netpbm PBM image text"
  | PGM_image_text -> "Netpbm PGM image text"
  | PPM_image_text -> "Netpbm PBM image text"
  | PBM_image_rawbits -> "Netpbm PBM \"rawbits\" image data"
  | PGM_image_rawbits -> "Netpbm PGM \"rawbits\" image data"
  | PPM_image_rawbits -> "Netpbm PPM \"rawbits\" image data"
  | PAM_image -> "Netpbm PAM image file"
  | ELF_32_bits_exe -> "ELF executable file (32 bits)"
  | ELF_64_bits_exe -> "ELF executable file (64 bits)"
  | OCAML_Cmi -> "Objective Caml CMI file"
  | OCAML_Cmo ->"Objective Caml CMO file"
  | OCAML_Cmx -> "Objective Caml CMX file"
  | OCAML_Cma -> "Objective Caml CMA file"
  | OCAML_Cmxa -> "Objective Caml CMXA file"
  | Unknown_format -> "Unknown format"
;;
}

let opt_space = (' ' | '\n' | '\t')*


(* Attention, les codes d'chappement de caractres *)
(* sont en DCIMAL, sur 3 chiffres (syntaxe Ocaml). *)
rule file = parse
  | "%!PS-Adobe-" { Postscript }
  | "%PDF-"       { Pdf }
  | _ _ ("-lh0-" | "-lh1-" | "-lz4-" | "-lz5-" | "-lzs-" | "-lhd-" | "-lh2-" | "-lh3-" | "-lh4-" | "-lh5-") { Lha }
  | '\247' '\002' { Dvi }
  | "\\input" | "\\section" | "\\setlength" | "\\documentstyle" | "\\chapter" { Latex }
  | "Rar!" { Rar }
  | "PK" '\003' '\004' { Zip }
  | '\254' '\055' '\000' '\035' { MS_Office_doc }
  | '\208' '\207' '\017' '\224' '\161' '\177' { MS_Office_doc }
  | '\202' '\254' '\186' '\190' { Java_bytecode }
  | '\031' '\139' { Gzip }
  | '\031' '\157' { Compress }
  | '\096' '\234' { Arj }
  | "MZ" { Msdos_exe }
  | "GIF8" { Gif }
  | "P1 " { PBM_image_text }
  | "P2 " { PGM_image_text }
  | "P3 " { PPM_image_text }
  | "P4 " { PBM_image_rawbits }
  | "P5 " { PGM_image_rawbits }
  | "P6 " { PPM_image_rawbits }
  | "P7 " { PAM_image }
  | '\137' "PNG" { Png }
  | '\255' '\216' { Jpeg }
  | "\127ELF\001" { ELF_32_bits_exe }
  | "\127ELF\002" { ELF_64_bits_exe }
  | "BM" { Bmp }
  | "/* XPM */" { Xpm }
  | "RIFF" { Riff_wav }
  | "EMOD" { Mod }
  | '\046' '\114' '\097' '\253' { Realaudio }
  | "FORM" { Iff }
  | "#FIG" { Fig }
  | "ARF_BEGARF" { Phigs }
  | "BEGMF" { Cgm }
  | "SIT!" { Sit }
  | "{\\rtf" { Rtf }
  | opt_space "/*" { C_H }
  | "#" (' ' | '\t')* ("define" | "include" | "ifdef" | "indef")  { C_H }
  | opt_space ("<HTML>" | "<TITLE>" | "<BODY>" | "<!DOCTYPE" | "<BASE" | "<HEAD>" | "<html>" | "<title>" | "<body>" | "<!doctype" | "<base" | "<head>" | "<!--" ) { Html }
  | "Caml1999I" { OCAML_Cmi }
  | "Caml1999O" { OCAML_Cmo }
  | "Caml1999Y" { OCAML_Cmx }
  | "Caml1999A" { OCAML_Cma }
  | "Caml1999Z" { OCAML_Cmxa }
  | _ { Unknown_format }

