
{
(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


module CKeywordElem = struct type t = string let compare = compare end ;;
module CKeywordSet = Set.Make (CKeywordElem) ;;
let keywords = ref CKeywordSet.empty ;;

(* Initialise la liste de mots clefs  ignorer. En fait a *)
(* revient  avoir une stop-list secondaire.               *)
List.iter (fun kw -> keywords := CKeywordSet.add kw !keywords)
    [ (* Mots clef du C eux-mmes. *)
      "auto";"break";"case";"char";"const";"continue";"default";"do";
      "double";"else";"enum";"extern";"float";"for";"goto";"if";"int";
      "long";"register";"return";"short";"signed";"sizeof";"static"; "struct";
      "switch";"typedef";"union";"unsigned";"void";"volatile"; "while" ;
      (* Pour le prprocesseur C. *)
      "define";"ifdef";"ifndef";"include"]
;;

let one_line_comment_started = ref false ;;
}

let lowercase = ['a'-'z' '\223'-'\246' '\248'-'\255' '_']
let uppercase = ['A'-'Z' '\192'-'\214' '\216'-'\222']
let digit = ['0'-'9']

rule file = parse
  | "//"
      {
       one_line_comment_started := true ;
       (Token.Markup Markup.Comment_start, Lexing.lexeme_start lexbuf)
      }
  | '\n'
      {
       (* Si un commentaire  1 ligne a t ouvert, alors a le termine *)
       if !one_line_comment_started then
	 begin
	 one_line_comment_started := false ;
	 (Token.Markup Markup.Comment_stop, Lexing.lexeme_start lexbuf)
	 end
       else file lexbuf
      }
  | "/*"
      {
       (Token.Markup Markup.Comment_start, Lexing.lexeme_start lexbuf)
      }
  | "*/"
      {
       (Token.Markup Markup.Comment_stop, Lexing.lexeme_start lexbuf)
      }
  | (lowercase | uppercase)+ (lowercase | uppercase | digit)*
      {
	let word = String.lowercase (Lexing.lexeme lexbuf) in
	(* Handle stop-list *)
	if Stoplist.mem word !GlobalStoplist.stop_list ||
           CKeywordSet.mem word !keywords
	then file lexbuf
	else (Token.Text word, Lexing.lexeme_start lexbuf)
      }
  | eof
      {
	(* Return the ending offset of the file includes the eof character. *)
	(* == Taille du fichier *)
	(Token.Markup Markup.Eof, Lexing.lexeme_start lexbuf)
      }
  | _
      {
	file lexbuf
      }
