
{
(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)

}

let lowercase = ['a'-'z' '\223'-'\246' '\248'-'\255' '_']
let uppercase = ['A'-'Z' '\192'-'\214' '\216'-'\222']
let digit = ['0'-'9']
let space = [' ' '\n' '\t']

rule file = parse
  | "<TITLE" | "<title"
      { (Token.Markup Markup.Title_start, Lexing.lexeme_start lexbuf) }
  | "</TITLE" | "</title"
      { (Token.Markup Markup.Title_stop, Lexing.lexeme_start lexbuf) }

  | ("<H1" | "<h1") space* ">"
      { (Token.Markup Markup.Section_start, Lexing.lexeme_start lexbuf) }
  | ("</H1" | "</h1") space* ">"
      { (Token.Markup Markup.Section_stop, Lexing.lexeme_start lexbuf) }

  | ("<H2" | "<h2") space* ">"
      { (Token.Markup Markup.Subsection_start, Lexing.lexeme_start lexbuf) }
  | ("</H2" | "</h2") space* ">"
      { (Token.Markup Markup.Subsection_stop, Lexing.lexeme_start lexbuf) }

  | ("<H3" | "<h3") space* ">"
      { (Token.Markup Markup.Subsubsection_start, Lexing.lexeme_start lexbuf) }
  | ("</H3" | "</h3") space* ">"
      { (Token.Markup Markup.Subsubsection_stop, Lexing.lexeme_start lexbuf) }

  | "<TABLE" | "<table"
      {	(Token.Markup Markup.Table_start, Lexing.lexeme_start lexbuf) }
  | "</TABLE" | "</table"
      { (Token.Markup Markup.Table_stop, Lexing.lexeme_start lexbuf) }

  | ("<B" | "<b") space* ">"
      { (* Gras dur *)
	(Token.Markup Markup.Relief_start, Lexing.lexeme_start lexbuf) }
  | ("</B" | "</b") space* ">"
      { (Token.Markup Markup.Relief_stop, Lexing.lexeme_start lexbuf) }
  | ("<STRONG" | "<strong") space* ">"
      { (* Texte pais (souvent gras) *)
	(Token.Markup Markup.Relief_start, Lexing.lexeme_start lexbuf) }
  | ("</STRONG" | "</strong") space* ">"
      { (Token.Markup Markup.Relief_stop, Lexing.lexeme_start lexbuf) }
  | ("<EM" | "<em") space* ">"
      { (* Mise en valeur (souvent italique) *)
	(Token.Markup Markup.Relief_start, Lexing.lexeme_start lexbuf) }
  | ("</EM" | "</em") space* ">"
      { (Token.Markup Markup.Relief_stop, Lexing.lexeme_start lexbuf) }
  | ("<I" | "<i") space* ">"
      { (* Italique dur *)
	(Token.Markup Markup.Relief_start, Lexing.lexeme_start lexbuf) }
  | ("</I" | "</i") space* ">"
      { (Token.Markup Markup.Relief_stop, Lexing.lexeme_start lexbuf) }

  | "<MENU>" | "<menu>"
      {	(Token.Markup Markup.Enum_start, Lexing.lexeme_start lexbuf) }
  | "</MENU>" | "</menu>"
      {	(Token.Markup Markup.Enum_stop, Lexing.lexeme_start lexbuf) }

  | (lowercase | uppercase)+ (lowercase | uppercase | digit)*
      {
	let word = String.lowercase (Lexing.lexeme lexbuf) in
	(* Handle stop-list *)
	if Stoplist.mem word !GlobalStoplist.stop_list then file lexbuf
	else (Token.Text word, Lexing.lexeme_start lexbuf)
      }
  | eof
      {
	(* Return the ending offset of the file includes the eof character. *)
	(* == Taille du fichier *)
	(Token.Markup Markup.Eof, Lexing.lexeme_start lexbuf)
      }
  | _
      {
	file lexbuf
      }
