
{
(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


open Token ;;
open Markup ;;

let one_line_comment_started = ref false ;;
}


let lowercase = ['a'-'z' '\223'-'\246' '\248'-'\255' '_']
let uppercase = ['A'-'Z' '\192'-'\214' '\216'-'\222']
let digit = ['0'-'9']
let space = ['\t' ' ']
let return = ['\n']
let delim = (space* | return)
let ident = (lowercase | uppercase)+ (lowercase | uppercase | digit)*
let whateverbutbraces = ['\000'-'\122' '\124' '\126'-'\255']


rule file = parse
  | "%"
      {
       one_line_comment_started := true ;
       (Token.Markup Markup.Comment_start, Lexing.lexeme_start lexbuf)
     }

  | '\n'
      { (* Si un commentaire  1 ligne a t ouvert, alors a le termine *)
	if !one_line_comment_started then
	  begin
	  one_line_comment_started := false ;
	  (Token.Markup Markup.Comment_stop, Lexing.lexeme_start lexbuf)
	  end
	else file lexbuf
      }

  | "\\begin" space* ("{itemize}" | "{enumerate}")
      { (Token.Markup Markup.Enum_start, Lexing.lexeme_start lexbuf) }
  | "\\end" space* ("{itemize}" | "{enumerate}")
      { (Token.Markup Markup.Enum_stop, Lexing.lexeme_start lexbuf) }

  | "\\begin" space* ("{array}" | "{tabular}")
      { (Token.Markup Markup.Table_start, Lexing.lexeme_start lexbuf) }
  | "\\end" space* ("{array}" | "{tabular}")
      { (Token.Markup Markup.Table_stop, Lexing.lexeme_start lexbuf) }

  | "\\begin" space* "{" ident "}" | "\\end" space* "{" ident "}"
      { file lexbuf }

  | "\\cite" space* "{" whateverbutbraces* "}"

  | "\\" ident
      { file lexbuf }

  | ident
      {
       let word = String.lowercase (Lexing.lexeme lexbuf) in
       (* Handle stop-list *)
       if Stoplist.mem word !GlobalStoplist.stop_list then file lexbuf
       else (Text word, Lexing.lexeme_start lexbuf)
      }
  | eof
      { (* Return the ending offset of the file includes the eof character. *)
	(* == Taille du fichier *)
	(Markup Markup.Eof, Lexing.lexeme_start lexbuf)
      }
  | _
      { file lexbuf }
