(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


let main () =
  Arg.parse
    [("-o",
      Arg.String (fun s -> Create.output_filename := s),
      "Specify output name for the index") ;
     ("-i",
      Arg.String
	(fun pattern ->
	  Create.ignored_regexprs := pattern :: !Create.ignored_regexprs),
      "Ignore file names matching the given pattern during indexing") ;
     ("-s",
      Arg.String (fun sl_name ->
	Create.stoplist_filenames := sl_name :: !Create.stoplist_filenames),
      "Specify a stop-list (word to ignore during indexing)") ;
     ("-r",
      Arg.Unit (fun () -> Create.recursive_indexing := true),
      "Index recursively in directories") ;
     ("-R",
      Arg.Unit (fun () -> Create.recursive_indexing := true),
      "Index recursively in directories.
       Don't add a leading '/' after the ending directory name.") ;
     ("-g",
      Arg.Unit (fun () -> Create.guess_indexing := true),
      "Enable lexing guessing") ;
     ("-F",
      Arg.String (fun name ->
	Create.files_filenames := name :: !Create.files_filenames),
      "Specify a file containing file names to index") ;
     ("-f",
      Arg.Unit (fun () -> Create.forget_unknown := true),
      "Forget indexing files with unknown format.
       By default plain text lexing is used.
       Has effect only if -g option is enabled.") ]
    (fun filename ->
      Create.cmdline_filenames := filename :: !Create.cmdline_filenames)
    "Creates an index from files given as arguments.
     Usage: createindex [-s | -r | -R | -g | -F | -f | -o <file>] <files>" ;
  (* Maintenant on traite rellement les fichiers de la ligne de commande. *)
  Create.go () ;
  flush stderr ;
  exit 0
;;

main () ;;
