(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


(* Boolen disant si on doit afficher juste les mots de la base de donne *)
(* ou si on doit aussi afficher les positions de leurs occurences.        *)
let short_dump = ref false ;;



(* **************************
val dump_index : Index.t -> unit

Juste de l'affichage banal...
************************** *)
let dump_index { Index.map = map ; Index.data = data } =
  Printf.printf "INDEX FILE MAPPING:\n" ;
  Array.iter (fun ((start, stop), filename) ->
                Printf.printf "\t[%d : %d] : file '%s'\n" start stop filename)
             map ;
  Printf.printf "TOKEN POSITIONS:\n" ;
  if not !short_dump then
    Hashtbl.iter (fun token positions ->
                     Printf.printf "\ttoken '%s' positions : "
                                    (Token.string token) ;
                     Array.iter (fun p -> Printf.printf "%d " p) positions ;
	             Printf.printf "\n")
                 data
  else
    Hashtbl.iter (fun token positions ->
                    Printf.printf "%s\n" (Token.string token))
                 data ;
  flush stderr
;;



let main () =
  let indexname = ref "" in
  Arg.parse [("-s", Arg.Unit (fun () -> short_dump := true),
              "Only display words present in the index")]
    (fun fname -> indexname := fname)
    "Dumps a whole index structure.
     Usage: dumpindex [-s] <index file>" ;
  if !indexname = "" then Printf.printf "Please specify an index file.\n"
  else
    begin
    let input_file = open_in_bin !indexname in
    let index = ((input_value input_file) : Index.t) in
    dump_index index
    end ;
  flush stderr ;
  exit 0
;;
    

try main () with
 | Sys_error err_msg -> Printf.eprintf "Error: %s\n" err_msg
;;

