(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


(* *************************************************************************
   [Fonc] error_notify :
            < on_window_error_close_button_clicked : unit -> unit;
              set_window_error :
                Gui_glade_interface.top_window_error -> unit;
                .. > ->
                  string -> unit
   [Description] : Affichage d'une fentre d'erreur modale au dessus de la
       fentre principale retrouve via le 'callback_object' pass en
       argument. Un icone d'erreur est affich  gauche de cette fentre.
   [Args] :
     - callback_object : L'objet provenant de Gui_mine.all_callbacks et
         mmorisant le contexte 'Glade' de l'interface graphique.
     - string : Le message d'erreur  afficher dans la fentre.
   [Retour] :
     - unit.
   [Rem] : Exporte hors de ce module.
   ************************************************************************* *)
let error_notify callback_object msg =
  let window_error =
    new Gui_glade_interface.top_window_error callback_object in
  callback_object#set_window_error window_error ;
  window_error#error_label#set_text msg ;
  ignore
    (GtkBase.Widget.add_events 
       window_error#window_error#as_widget [`ALL_EVENTS]) ;
  window_error#window_error#set_position `CENTER ;
  window_error#window_error#show () ;
  (* Lier la pression sur la touche 'Entre'  la sortie. *)
  ignore
    (window_error#window_error#event#connect#key_press
      ~callback:
       (fun ev ->
	 if (GdkEvent.Key.keyval ev = GdkKeysyms._Return) then
	   begin
	   window_error#window_error#destroy () ;
	   (* Retourner true pour ne pas laisser le gestionnaire   *)
	   (* par dfaut faire son travail et ainsi signifier que  *)
	   (* l'on a nous mme bien trait totalement l'vnement. *)
	   false
	   end else true)) ;
  (* Lier la pression sur la touche 'Escape'  la sortie. *)
  ignore
    (window_error#window_error#event#connect#key_press
      ~callback:
       (fun ev ->
	 if (GdkEvent.Key.keyval ev = GdkKeysyms._Escape) then
	   begin
	   window_error#window_error#destroy () ;
	   (* Retourner true pour ne pas laisser le gestionnaire   *)
	   (* par dfaut faire son travail et ainsi signifier que  *)
	   (* l'on a nous mme bien trait totalement l'vnement. *)
	   false
	   end else true)) ;
  ignore
    (window_error#window_error#connect#destroy ~callback: GMain.Main.quit) ;
  GMain.Main.main ()
;;



(* *************************************************************************
   [Fonc] warning_notify :
            < on_window_error_close_button_clicked : unit -> unit;
              set_window_error :
                Gui_glade_interface.top_window_warning -> unit;
                .. > ->
                  string -> unit
   [Description] : Affichage d'une fentre d'avertissement modale au dessus
        de la fentre principale retrouve via le 'callback_object' pass en
       argument. Un icone d'avertissement est affich  gauche de cette
       fentre.
   [Args] :
     - callback_object : L'objet provenant de Gui_mine.all_callbacks et
         mmorisant le contexte 'Glade' de l'interface graphique.
     - string : Le message d'avertissement  afficher dans la fentre.
   [Retour] :
     - unit.
   [Rem] : Exporte hors de ce module.
   ************************************************************************* *)
let warning_notify callback_object msg =
  let window_warning =
    new Gui_glade_interface.top_window_warning callback_object in
  callback_object#set_window_warning window_warning ;
  window_warning#warning_label#set_text msg ;
  ignore
    (GtkBase.Widget.add_events 
       window_warning#window_warning#as_widget [`ALL_EVENTS]) ;
  window_warning#window_warning#set_position `CENTER ;
  window_warning#window_warning#show () ;
  (* Lier la pression sur la touche 'Entre'  la sortie. *)
  ignore
    (window_warning#window_warning#event#connect#key_press
      ~callback:
       (fun ev ->
	 if (GdkEvent.Key.keyval ev = GdkKeysyms._Return) then
	   begin
	   window_warning#window_warning#destroy () ;
	   (* Retourner true pour ne pas laisser le gestionnaire   *)
	   (* par dfaut faire son travail et ainsi signifier que  *)
	   (* l'on a nous mme bien trait totalement l'vnement. *)
	   false
	   end else true)) ;
  (* Lier la pression sur la touche 'Escape'  la sortie. *)
  ignore
    (window_warning#window_warning#event#connect#key_press
      ~callback:
       (fun ev ->
	 if (GdkEvent.Key.keyval ev = GdkKeysyms._Escape) then
	   begin
	   window_warning#window_warning#destroy () ;
	   (* Retourner true pour ne pas laisser le gestionnaire   *)
	   (* par dfaut faire son travail et ainsi signifier que  *)
	   (* l'on a nous mme bien trait totalement l'vnement. *)
	   false
	   end else true)) ;
  ignore
    (window_warning#window_warning#connect#destroy
       ~callback: GMain.Main.quit) ;
  GMain.Main.main ()
;;



(* *************************************************************************
   [Fonc] info_notify :
            < on_window_warning_close_button_clicked : unit -> unit;
              set_window_warning :
                Gui_glade_interface.top_window_info -> unit;
                .. > ->
                  string -> unit
   [Description] : Affichage d'une fentre d'information modale au dessus
        de la fentre principale retrouve via le 'callback_object' pass en
       argument. Un icone d'information est affich  gauche de cette
       fentre.
   [Args] :
     - callback_object : L'objet provenant de Gui_mine.all_callbacks et
         mmorisant le contexte 'Glade' de l'interface graphique.
     - string : Le message d'information a`afficher dans la fentre.
   [Retour] :
     - unit.
   [Rem] : Exporte hors de ce module.
   ************************************************************************* *)
let info_notify callback_object msg =
  let window_info =
    new Gui_glade_interface.top_window_info callback_object in
  callback_object#set_window_info window_info ;
  window_info#info_label#set_text msg ;
  ignore
    (GtkBase.Widget.add_events 
       window_info#window_info#as_widget [`ALL_EVENTS]) ;
  window_info#window_info#set_position `CENTER ;
  window_info#window_info#show () ;
  (* Lier la pression sur la touche 'Entre'  la sortie. *)
  ignore
    (window_info#window_info#event#connect#key_press
      ~callback:
       (fun ev ->
	 if (GdkEvent.Key.keyval ev = GdkKeysyms._Return) then
	   begin
	   window_info#window_info#destroy () ;
	   (* Retourner true pour ne pas laisser le gestionnaire   *)
	   (* par dfaut faire son travail et ainsi signifier que  *)
	   (* l'on a nous mme bien trait totalement l'vnement. *)
	   false
	   end else true)) ;
  (* Lier la pression sur la touche 'Escape'  la sortie. *)
  ignore
    (window_info#window_info#event#connect#key_press
      ~callback:
       (fun ev ->
	 if (GdkEvent.Key.keyval ev = GdkKeysyms._Escape) then
	   begin
	   window_info#window_info#destroy () ;
	   (* Retourner true pour ne pas laisser le gestionnaire   *)
	   (* par dfaut faire son travail et ainsi signifier que  *)
	   (* l'on a nous mme bien trait totalement l'vnement. *)
	   false
	   end else true)) ;
  ignore
    (window_info#window_info#connect#destroy ~callback: GMain.Main.quit) ;
  GMain.Main.main ()
;;
