(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


class customized_callbacks = object (self)
  inherit Gui_mine.all_callbacks
end



let main () =
  let indexname = ref "" in
  Arg.parse []
    (fun fname -> indexname := fname)
    "Usage: gui [<index file>]" ;
  let callbacks = new customized_callbacks in
  let accueil = new Gui_glade_interface.top_main_window callbacks in  
  (* Rajout  la main de la clist de resultat car non gre par MlGlade. *)
  let result_clist =
    GList.clist_poly
      ~packing: (accueil#scrolledwindow3#add_with_viewport)
      ~titles_show: true
      ~titles: ["File" ; "Starting position" ; "Ending position"] () in
  result_clist#set_column ~min_width: 550 ~resizeable: true 0 ;
  result_clist#set_column ~min_width: 115 ~resizeable: true 1 ;
  result_clist#set_column ~min_width: 115 ~resizeable: true 2 ;
  result_clist#set_selection_mode `SINGLE ;
  Guiglobal.result_clist := Some result_clist ;
  (* Mmorisation en global du context de   *)
  (* la statusbar de la fentre principale. *)
  let statusbar_context = accueil#main_statusbar#new_context "ubiest" in
  Guiglobal.main_statusbar_context := Some statusbar_context ;
  ignore
    (result_clist#connect#select_row
       ~callback: callbacks#click_on_row_result_binding) ;
  ignore
    (GtkBase.Widget.add_events accueil#main_window#as_widget [`ALL_EVENTS]) ;
  (* On charge les prfrences si elles existent. *)
  Preferences.load_prefs () ;
  (* On essaye d'ouvrir les fontes. *)
  let exact_font =
    (try
      Some
	(Gdk.Font.load_fontset
	   !Preferences.global_prefs.Preferences.exact_font_name)
    with Gpointer.Null ->
      Convenient.error_notify
	callbacks ("Unsupported font '" ^
		   !Preferences.global_prefs.Preferences.exact_font_name ^
		   "' for exact context highlight.") ;
      None) in
  let context_font =
    (try
      Some
	(Gdk.Font.load_fontset
	   !Preferences.global_prefs.Preferences.context_font_name)
    with Gpointer.Null ->
      Convenient.error_notify
	callbacks ("Unsupported font '" ^
		   !Preferences.global_prefs.Preferences.context_font_name ^
		     "' for extended context highlight.") ;
      None) in
  (* Mise  jour des prfrences globales. *)
  Preferences.global_prefs := {
    !Preferences.global_prefs with
      Preferences.exact_font = exact_font ;
      Preferences.context_font = context_font } ;
  (* Et on affiche enfin la fentre principale. *)
  ignore (accueil#main_window#show()) ;
  ignore (statusbar_context#push "No index loaded") ;
  if !indexname <> "" then
    begin
    try
      let input_file = open_in_bin !indexname in
      Guiglobal.global_index := Some ((input_value input_file) : Index.t) ;
      close_in input_file ;
      ignore (statusbar_context#pop ()) ;
      ignore (statusbar_context#push ("Current index is : " ^ !indexname))
    with
    | Sys_error err -> Convenient.error_notify callbacks err
    | Failure ("input_value: bad object") ->
	Convenient.error_notify callbacks "Invalid index file format."
    end ;
  GMain.Main.main ()
;;


    
ignore (Printexc.print main ()) ;;
