(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


let max_prof = 2 ;;
let words = ref ([||] : string array) ;;



let random_token () =
  let w = !words.(Random.int (Array.length !words)) in
  Token.Text w
;;



let rec random_request current_prof =
  if current_prof = max_prof then Request.Simple (random_token ())
  else
    match Random.int 8 with
     | 0 -> Request.Simple (random_token ())
     | 1 -> Request.And (random_request (current_prof + 1),
			 random_request (current_prof + 1))
     | 2 -> Request.Or (random_request (current_prof + 1),
			random_request (current_prof + 1))
     | 3 -> Request.Followed_by (random_request (current_prof + 1),
				 random_request (current_prof + 1))
     | 4 -> Request.Contained_in (random_request (current_prof + 1),
				  random_request (current_prof + 1))
     | 5 -> Request.Containing (random_request (current_prof + 1),
				random_request (current_prof + 1))
     | 6 -> Request.Not_contained_in (random_request (current_prof + 1),
				      random_request (current_prof + 1))
     | 7 -> Request.Not_containing (random_request (current_prof + 1),
				    random_request (current_prof + 1))
     | _ -> assert false
;;



let load_words f =
  let file = open_in f in
  let go_on = ref true in
  let tmp = ref ([] : string list) in
  while !go_on do
    try
      let w = input_line file in
      tmp := w :: !tmp
    with End_of_file -> go_on := false
  done ;
  close_in file ;
  words := Array.of_list !tmp
;;



let go index =
  while true do
    let request = random_request 0 in
    (match Request.driver request index with
     | [] -> Printf.eprintf "0"      (* Rsultat de requte vide *)
     | _ -> Printf.eprintf ".") ;    (* Rsultat non vide *)
    flush stderr
  done
;;



let main () =
  (* Init du gnrateur de nombres alatoires *)
  Random.init (Unix.getpid ()) ;
  Arg.parse [("-w", Arg.String (fun f -> load_words f),
              "Specifies list of word to build requests")]
    (fun indexname ->
      let input_file = open_in_bin indexname in
      let index = ((input_value input_file) : Index.t) in
      go index)
    "Just monkey test access functions"
;;

main () ;;
