{
(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


open Parserequest

exception Lexical_error ;;

let initial_string_buffer = String.create 256 ;;
let string_buff = ref initial_string_buffer ;;
let string_index = ref 0 ;;

let reset_string_buffer () =
  string_buff := initial_string_buffer;
  string_index := 0 ;;

let store_string_char c =
  if !string_index >= String.length (!string_buff) then begin
    let new_buff = String.create (String.length (!string_buff) * 2) in
      String.blit (!string_buff) 0 new_buff 0 (String.length (!string_buff)) ;
      string_buff := new_buff
  end ;
  String.unsafe_set (!string_buff) (!string_index) c ;
  incr string_index ;;

let get_stored_string () =
  let s = String.sub (!string_buff) 0 (!string_index) in
  string_buff := initial_string_buffer ;
  s ;;

(* To store the position of the beginning of a string *)
let string_start_pos = ref 0 ;;
}


let identchar = 
  ['A'-'Z' 'a'-'z' '_' '\192'-'\214' '\216'-'\246' '\248'-'\255' '\'']
let digit = ['0'-'9']
let blank = [' ' '\009' '\012']
let return = [ '\010' '\013']


rule request = parse
  | "@comment"     { COMMENT }
  | "@title"       { TITLE }
  | "@section"     { SECTION }
  | "@ssection"    { SUBSECTION }
  | "@sssection"   { SUBSUBSECTION }
  | "@table"       { TABLE }
  | "@relief"      { RELIEF }
  | "@enum"        { ENUM }
  | "&"      { AMPERSAND }
  | "|"      { BAR }
  | return   { RETURN }
  | "("      { LPAREN }
  | ")"      { RPAREN }
  | "<>"     { LTGT }
  | "~<"     { NOTLT }
  | "~>"     { NOTGT }
  | "<"      { LT }
  | ">"      { GT }
  | "\""     { reset_string_buffer() ;
	       let string_start = Lexing.lexeme_start lexbuf in
	       string_start_pos := string_start ;
	       string lexbuf ;
	       lexbuf.Lexing.lex_start_pos <-
		        string_start - lexbuf.Lexing.lex_abs_pos ;
               TOKEN (get_stored_string ())
             }
  | (identchar)+ (identchar | digit)*
             { TOKEN (Lexing.lexeme lexbuf) }
  | blank+   { request lexbuf }
  | _        { raise Lexical_error }


and string = parse
    '"'
      { () }
  | _
      { store_string_char(Lexing.lexeme_char lexbuf 0);
        string lexbuf }
