(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)


let quotation = ref false ;;



(* **************************
val display_result : Index.t -> Gclist.extent list -> unit

Affiche les infos relatives aux occurences satisfaisant la requte.
************************** *)
let display_result index gclist =
  Printf.eprintf "Found %d occurence(s)\n" (List.length gclist) ;
  (* A ne pas oublier avant chaque nouvelle requte, *)
  (* sinon on pte les invariants du file-mapping !  *)
  Filemap.map_extend_init () ;
  (* Maintenant on peut appeler 'map_extend_on_file' pour toutes *)
  (* les occurrences de la requte courante.                     *)
  List.iter (fun extent ->
              let ((fstart, fstop), filename) =
		Filemap.map_extend_on_file index extent in
	      Printf.eprintf "\tIn file '%s' at characters [%d : %d]\n"
		             filename fstart fstop ;
	      (* Mentionne l'extent trouv si ncessaire *)
	      if !quotation then
		begin
		try
		  let file = open_in filename in
		  seek_in file fstart ;
		  let quote_size = (fstop - fstart) + 1 in
		  let quote = String.create quote_size in
		  let _ = input file quote 0 quote_size in
		  close_in file ;
		  Printf.eprintf "--Begin quote\n%s\n--End quote\n" quote
		with Sys_error msg ->
		  Printf.eprintf "Can't quote because: %s\n" msg
		end)
             gclist ;
  flush stderr
;;



(* **************************
val question_index : Index.t -> unit

Mainloop d'interrogation.
************************** *)
let question_index index =
  let lexbuf = Lexing.from_channel stdin in
  while true do
    try
      Printf.eprintf "?> " ;
      flush stderr ;
      let request = Parserequest.request Lexrequest.request lexbuf in
      if Request.verify_request index.Index.stoplist request then
	begin
	let gclist = Request.driver request index in
	display_result index gclist
	end
      else
	Printf.eprintf
	  "Error : request contains words ignored during indexing\n"
    with
     | Lexrequest.Lexical_error ->
	 Printf.eprintf "Lexical error\n" ; flush stderr
     | Parsing.Parse_error ->
	 Printf.eprintf "Syntax error\n" ; flush stderr
  done
;;



let main () =
  let indexname = ref "" in
  Arg.parse
    [("-q", Arg.Unit (fun () -> quotation := true), "Quote found extents")]
    (fun fname -> indexname := fname)
    "Asks requests in the index.
     Usage: query [-q] <index file>" ;
  if !indexname = "" then Printf.eprintf "Please specify an index file.\n"
  else
    begin
    let input_file = open_in_bin !indexname in
    let index = ((input_value input_file) : Index.t) in
    question_index index ;
    flush stderr
    end ;
  exit 0
;;


try main () with
 | Sys_error err_msg -> Printf.eprintf "Error: %s\n" err_msg
;;
