%{
(********************************************************************** *)
(*                                                                      *)
(*                        Ubi Est - Indexing package                    *)
(*                                                                      *)
(*  Copyright Franois Pessaux 1999-2007.                               *)
(*  Distributed according to the LICENCE file contained in the archive. *)
(*                                                                      *)
(************************************************************************)

%}

/* Tokens */
%token AMPERSAND
%token BAR
%token MINUS
%token RETURN
%token <string> TOKEN
%token LPAREN
%token RPAREN
%token LTGT
%token LT
%token GT
%token NOTLT
%token NOTGT

%token COMMENT
%token TITLE
%token SECTION
%token SUBSECTION
%token SUBSUBSECTION
%token TABLE
%token RELIEF
%token ENUM

%right BAR                        /* | */
%right AMPERSAND                  /* & */
%left GT LT LTGT

/* Entry points */
%start request
%type <Request.request> request

%%

/* Entry point */
request:
    subrequest RETURN                 { $1 }
;

subrequest:
   TOKEN                              { Request.Simple (Token.Text $1) }
 | LPAREN subrequest RPAREN           { $2 }
 | COMMENT subrequest
     {
       let r1 = Request.Simple (Token.Markup Markup.Comment_start) in
       let r2 = Request.Simple (Token.Markup Markup.Comment_stop) in
       let r3 = Request.Followed_by (r1, r2) in
       Request.Contained_in ($2, r3)
     }
 | TITLE subrequest
     {
       let r1 = Request.Simple (Token.Markup Markup.Title_start) in
       let r2 = Request.Simple (Token.Markup Markup.Title_stop) in
       let r3 = Request.Followed_by (r1, r2) in
       Request.Contained_in ($2, r3)
     }
 | SECTION subrequest
     {
       let r1 = Request.Simple (Token.Markup Markup.Section_start) in
       let r2 = Request.Simple (Token.Markup Markup.Section_stop) in
       let r3 = Request.Followed_by (r1, r2) in
       Request.Contained_in ($2, r3)
     }
 | SUBSECTION subrequest
     {
       let r1 = Request.Simple (Token.Markup Markup.Subsection_start) in
       let r2 = Request.Simple (Token.Markup Markup.Subsection_stop) in
       let r3 = Request.Followed_by (r1, r2) in
       Request.Contained_in ($2, r3)
     }
 | SUBSUBSECTION subrequest
     {
       let r1 = Request.Simple (Token.Markup Markup.Subsubsection_start) in
       let r2 = Request.Simple (Token.Markup Markup.Subsubsection_stop) in
       let r3 = Request.Followed_by (r1, r2) in
       Request.Contained_in ($2, r3)
     }
 | TABLE subrequest
     {
       let r1 = Request.Simple (Token.Markup Markup.Table_start) in
       let r2 = Request.Simple (Token.Markup Markup.Table_stop) in
       let r3 = Request.Followed_by (r1, r2) in
       Request.Contained_in ($2, r3)
     }
 | RELIEF subrequest
     {
       let r1 = Request.Simple (Token.Markup Markup.Relief_start) in
       let r2 = Request.Simple (Token.Markup Markup.Relief_stop) in
       let r3 = Request.Followed_by (r1, r2) in
       Request.Contained_in ($2, r3)
     }
 | ENUM subrequest
     {
       let r1 = Request.Simple (Token.Markup Markup.Enum_start) in
       let r2 = Request.Simple (Token.Markup Markup.Enum_stop) in
       let r3 = Request.Followed_by (r1, r2) in
       Request.Contained_in ($2, r3)
     }
 | subrequest AMPERSAND subrequest    { Request.And ($1, $3) }
 | subrequest BAR subrequest          { Request.Or ($1, $3) }
 | subrequest LTGT subrequest         { Request.Followed_by ($1, $3) }
 | subrequest LT subrequest           { Request.Contained_in ($1, $3) }
 | subrequest GT subrequest           { Request.Containing ($1, $3) }
 | subrequest NOTLT subrequest        { Request.Not_contained_in ($1, $3) }
 | subrequest NOTGT subrequest        { Request.Not_containing ($1, $3) }
%%
